#ifndef GMIME_CONTENT__H
#define GMIME_CONTENT__H

#include "gmime-part.h"

/**
***  Content Encoding
**/

typedef enum
{
	GMIME_PART_ENCODING_DEFAULT,
	GMIME_PART_ENCODING_7BIT,
	GMIME_PART_ENCODING_8BIT,
	GMIME_PART_ENCODING_BASE64,
	GMIME_PART_ENCODING_QUOTEDPRINTABLE,
	GMIME_PART_NUM_ENCODINGS
}
GMimePartEncodingType;

const gchar *g_mime_part_encoding_to_string (GMimePartEncodingType encoding);

GMimePartEncodingType g_mime_part_encoding_from_string (const gchar *encoding);

/**
***  Read Iterator
**/

struct _GMimeContentIterator
{
	/* virtual functions */
	gint (*step_func)(struct _GMimeContentIterator*, guchar*, guint);
	gboolean (*readln_func)(struct _GMimeContentIterator*, GByteArray* line);
	void (*destroy_func)(struct _GMimeContentIterator*);

	/* implementation details */
	gpointer data;
};

typedef struct _GMimeContentIterator GMimeContentIterator;

gint      g_mime_content_iterator_step      (GMimeContentIterator  * it,
                                             guchar                * buf,
                                             guint                   maxlen);

gboolean  g_mime_content_iterator_readln    (GMimeContentIterator  * it,
                                             GByteArray            * setme);

void      g_mime_content_iterator_destroy   (GMimeContentIterator  * it);



/**
***  Part Contents
**/

struct _GMimeContent
{
	/* virtual functions */
	void (*content_set_begin_func)(struct _GMimeContent*);
	void (*content_append_func)(struct _GMimeContent*, const guchar*, guint);
	void (*content_append_pre_encoded_func)(struct _GMimeContent*, const guchar*, guint, GMimePartEncodingType);
	void (*content_set_end_func)(struct _GMimeContent*);
	GMimeContentIterator* (*content_get_iterator_func)(const struct _GMimeContent*);

	/* implementation details */
	gpointer data;
	GDestroyNotify data_destroy;

	/* public data */
	guint len;
	guint append_save;
	guint append_state;
	GMimePartEncodingType encoding;
};

typedef struct _GMimeContent GMimeContent;



/**
***  Life Cycle
**/

GMimeContent*           g_mime_content_file_new                 (void);

GMimeContent*           g_mime_content_array_new                (void);

void                    g_mime_content_destroy                  (GMimeContent             * content);

/**
***  Encoding
**/

void                    g_mime_content_set_encoding             (GMimeContent             * content,
                                                                 GMimePartEncodingType      encoding);

GMimePartEncodingType   g_mime_content_get_encoding             (GMimeContent             * content);

/**
***  Read Contents
**/

void                    g_mime_content_get_all                  (const GMimeContent       * content,
                                                                 GByteArray               * setme);

GMimeContentIterator*   g_mime_content_get_iterator             (const GMimeContent       * content);


/**
***  Set Contents
**/

void                    g_mime_content_set_contents             (GMimeContent             * content,
                                                                 const guchar             * bytes,
                                                                 guint                      byte_qty);

void                    g_mime_content_set_all                  (GMimeContent             * content,
                                                                 const guchar             * bytes,
                                                                 guint                      byte_qty);

void                    g_mime_content_set_all_pre_encoded      (GMimeContent             * content,
                                                                 const guchar             * encoded_bytes,
                                                                 guint                      encoded_len,
                                                                 GMimePartEncodingType      encoded_type);


void                    g_mime_content_set_begin                (GMimeContent             * content);

void                    g_mime_content_append                   (GMimeContent             * content,
                                                                 const guchar             * bytes,
                                                                 guint                      bye_qty);

void                    g_mime_content_append_pre_encoded       (GMimeContent             * content,
                                                                 const guchar             * encoded_bytes,
                                                                 guint                      encoded_len,
                                                                 GMimePartEncodingType      encodedtype);

void                    g_mime_content_set_end                  (GMimeContent             * content);


#endif
